<?php
namespace FiloBlu\Refilo\Model;
use FiloBlu\Refilo\Api\CustomerGroupManagmentInterface;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Customer\Api\Data\CustomerInterface;
use Psr\Log\LoggerInterface;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Customer\Model\CustomerRegistry;
use Magento\Customer\Model\CustomerFactory as CustomerModelFactory;

class CustomerGroupManagment implements CustomerGroupManagmentInterface
{
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;
    /**
     * @var CustomerRegistry
     */
    private $customerRegistry;
    /**
     * @var CustomerModelFactory
     */
    private $customerModelFactory;

    /**
     * CustomerGroupManagment constructor.
     * @param CustomerRepositoryInterface $customerRepository
     * @param LoggerInterface $logger
     * @param CartRepositoryInterface $cartRepository
     * @param CustomerRegistry $customerRegistry
     * @param CustomerModelFactory $customerModelFactory
     */
    public function __construct(
        CustomerRepositoryInterface $customerRepository,
        LoggerInterface $logger,
        CartRepositoryInterface  $cartRepository,
        CustomerRegistry $customerRegistry,
        CustomerModelFactory $customerModelFactory){
        $this->customerRepository = $customerRepository;
        $this->logger = $logger;
        $this->cartRepository = $cartRepository;
        $this->customerRegistry = $customerRegistry;
        $this->customerModelFactory = $customerModelFactory;
    }

    /**
     * @param int $customerId
     * @param int $customerGroupId
     * @return CustomerInterface
     */
    public function updateCustomerGroup(int $customerId, int $customerGroupId)
    {
        try{
            try{
                $customer = $this->customerRepository->getById($customerId);
            }catch (\Exception $e){
                $customer = $this->pushCustomerIntoRegistry($customerId);
            }catch (\Throwable $t){
                $customer = $this->pushCustomerIntoRegistry($customerId);
            }

            if($customer->getGroupId() != $customerGroupId){
                $cart = $this->cartRepository->getActiveForCustomer($customerId,[$customer->getStoreId()]);
                $customer->setGroupId($customerGroupId);
                $this->customerRepository->save($customer);
                $cart->setCustomer($customer);
                $this->cartRepository->save($cart);
            }
            return $customer;
        }catch (\Exception $e){
            $this->logger->critical($e->getMessage());
        }catch (\Throwable $t){
            $this->logger->critical($t->getMessage());
        }
    }

    /**
     * @param int $customerId
     * @return CustomerRegistry
     */
    protected function pushCustomerIntoRegistry(int $customerId){
       return  $this->customerRegistry->push($this->customerModelFactory->create()->load($customerId));
    }
}
